"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "createCanvas", {
  enumerable: true,
  get: function () {
    return _WebRtcPeerCore.createCanvas;
  }
});
exports.default = exports.WebRtcPeer = void 0;

var _uaParserJs = _interopRequireDefault(require("ua-parser-js"));

var _wrtc = require("wrtc");

var _WebRtcPeerCore = require("./WebRtcPeerCore");

var _window$navigator;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classPrivateMethodGet(receiver, privateSet, fn) { if (!privateSet.has(receiver)) { throw new TypeError("attempted to get private field on non-instance"); } return fn; }

function _classPrivateFieldGet(receiver, privateMap) { var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "get"); return _classApplyDescriptorGet(receiver, descriptor); }

function _classApplyDescriptorGet(receiver, descriptor) { if (descriptor.get) { return descriptor.get.call(receiver); } return descriptor.value; }

function _classPrivateFieldSet(receiver, privateMap, value) { var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "set"); _classApplyDescriptorSet(receiver, descriptor, value); return value; }

function _classExtractFieldDescriptor(receiver, privateMap, action) { if (!privateMap.has(receiver)) { throw new TypeError("attempted to " + action + " private field on non-instance"); } return privateMap.get(receiver); }

function _classApplyDescriptorSet(receiver, descriptor, value) { if (descriptor.set) { descriptor.set.call(receiver, value); } else { if (!descriptor.writable) { throw new TypeError("attempted to set read only private field"); } descriptor.value = value; } }

// Somehow, the UAParser constructor gets an empty window object.
// We need to pass the user agent string in order to get information
const ua = typeof window !== 'undefined' && ((_window$navigator = window.navigator) === null || _window$navigator === void 0 ? void 0 : _window$navigator.userAgent) || '';
const parser = new _uaParserJs.default(ua);
const {
  name
} = parser.getBrowser();
const usePlanB = name === 'Chrome' || name === 'Chromium';
/**
 * Wrapper object of an {RTCPeerConnection}. This object is aimed to simplify
 * the development of WebRTC-based applications.
 */

var _localVideo = /*#__PURE__*/new WeakMap();

var _remoteVideo = /*#__PURE__*/new WeakMap();

var _setRemoteVideo = /*#__PURE__*/new WeakMap();

var _setLocalVideo = /*#__PURE__*/new WeakSet();

class WebRtcPeer extends _WebRtcPeerCore.WebRtcPeerCore {
  /**
   * @constructor module:kurentoUtils.WebRtcPeer
   *
   * @param {String} mode Mode in which the PeerConnection will be configured.
   *  Valid values are: 'recv', 'send', and 'sendRecv'
   * @param {Object} [options]
   * @param {MediaStream} [options.audioStream] Stream to be used as second
   *  source (typically for audio) for localVideo and to be added as stream to
   *  the {RTCPeerConnection}
   * @param {HTMLVideoElement} [options.localVideo] Video tag for the local
   *  stream
   * @param {HTMLVideoElement} [options.remoteVideo] Video tag for the remote
   *  stream
   * @param {MediaStream} [options.videoStream] Stream to be used as primary
   *  source (typically video and audio, or only video if combined with
   *  audioStream) for localVideo and to be added as stream to the
   *  {RTCPeerConnection}
   */
  constructor(mode, options, callback) {
    if (options instanceof Function) {
      callback = options;
      options = undefined;
    }

    const {
      localVideo,
      remoteVideo,
      ...coreOptions
    } = options || {};
    super(mode, {
      usePlanB,
      ...coreOptions
    }, callback);

    _setLocalVideo.add(this);

    _localVideo.set(this, {
      writable: true,
      value: void 0
    });

    _remoteVideo.set(this, {
      writable: true,
      value: void 0
    });

    _setRemoteVideo.set(this, {
      writable: true,
      value: () => {
        if (!_classPrivateFieldGet(this, _remoteVideo)) return;
        const stream = new _wrtc.MediaStream();

        for (const {
          track
        } of this.peerConnection.getReceivers()) stream.addTrack(track);

        _classPrivateFieldGet(this, _remoteVideo).pause();

        _classPrivateFieldGet(this, _remoteVideo).srcObject = stream;

        _classPrivateFieldGet(this, _remoteVideo).load();
      }
    });

    _classPrivateFieldSet(this, _localVideo, localVideo);

    _classPrivateFieldSet(this, _remoteVideo, remoteVideo);

    this.on('setRemoteVideo', _classPrivateFieldGet(this, _setRemoteVideo));
    this.on('setLocalVideo', _classPrivateMethodGet(this, _setLocalVideo, _setLocalVideo2));
  } //
  // Public API
  //

  /**
   * @member {(external:ImageData|undefined)} currentFrame
   */


  get currentFrame() {
    let video = _classPrivateFieldGet(this, _remoteVideo);

    if (!video) {
      // We have a remote stream but we didn't set a remoteVideo tag
      const receivers = this.peerConnection.getReceivers();
      if (!receivers.length) throw new Error('No remote video stream available');
      const stream = new _wrtc.MediaStream();

      for (const {
        track
      } of receivers) stream.addTrack(track);

      video = document.createElement('video');
      video.srcObject = stream;
    } else if (video.readyState < video.HAVE_CURRENT_DATA) throw new Error('No remote video stream data available');

    const canvas = (0, _WebRtcPeerCore.createCanvas)(video.videoWidth, video.videoHeight);
    canvas.getContext('2d').drawImage(video, 0, 0);

    if (!_classPrivateFieldGet(this, _remoteVideo)) {
      video.pause();
      video.srcObject = null;
      video.load();
    }

    return canvas;
  }

  get localVideo() {
    return _classPrivateFieldGet(this, _localVideo);
  }

  get remoteVideo() {
    return _classPrivateFieldGet(this, _remoteVideo);
  }
  /**
   * @description This method frees the resources used by WebRtcPeer.
   *
   * @function module:kurentoUtils.WebRtcPeer.prototype.dispose
   */


  dispose() {
    if (_classPrivateFieldGet(this, _localVideo)) {
      _classPrivateFieldGet(this, _localVideo).pause();

      _classPrivateFieldGet(this, _localVideo).srcObject = null;

      _classPrivateFieldGet(this, _localVideo).load();

      _classPrivateFieldGet(this, _localVideo).muted = false;
    }

    if (_classPrivateFieldGet(this, _remoteVideo)) {
      _classPrivateFieldGet(this, _remoteVideo).pause();

      _classPrivateFieldGet(this, _remoteVideo).srcObject = null;

      _classPrivateFieldGet(this, _remoteVideo).load();
    }

    super.dispose();
  } //
  // Deprecated public APIs
  //

  /**
   * Get local session descriptor
   *
   * @deprecated use `WebRtcPeerCore.peerConnection.localDescription` instead
   */

  /* istanbul ignore next */


  getLocalSessionDescriptor() {
    return this.peerConnection.localDescription;
  }
  /**
   * Get local stream
   *
   * @deprecated use directly `WebRtcPeerCore.getSender()`
   */

  /* istanbul ignore next */


  getLocalStream(index) {
    const stream = new _wrtc.MediaStream();

    for (const {
      track
    } of this.getSender(index)) stream.addTrack(track);

    return stream;
  }
  /**
   * Get remote session descriptor
   *
   * @deprecated use `WebRtcPeerCore.peerConnection.remoteDescription` instead
   */

  /* istanbul ignore next */


  getRemoteSessionDescriptor() {
    return this.peerConnection.remoteDescription;
  }
  /**
   * Get remote stream
   *
   * @deprecated use directly `WebRtcPeerCore.getReceiver()`
   */

  /* istanbul ignore next */


  getRemoteStream(index) {
    const stream = new _wrtc.MediaStream();

    for (const {
      track
    } of this.getSender(index)) stream.addTrack(track);

    return stream;
  }
  /**
   * Set the local video to the current local video stream
   *
   * @deprecated Local video is automatically set, this is a no-op, don't use it
   */

  /* istanbul ignore next */


  showLocalVideo() {} //
  // Private API
  //


}

exports.WebRtcPeer = WebRtcPeer;

function _setLocalVideo2() {
  if (!_classPrivateFieldGet(this, _localVideo)) return;
  _classPrivateFieldGet(this, _localVideo).srcObject = this.videoStream;
  _classPrivateFieldGet(this, _localVideo).muted = true;
}

var _default = WebRtcPeer; // https://github.com/Automattic/node-canvas/issues/487

exports.default = _default;