"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "createCanvas", {
  enumerable: true,
  get: function () {
    return _canvas.createCanvas;
  }
});
exports.WebRtcPeerCore = exports.default = void 0;

var _events = _interopRequireDefault(require("events"));

require("webrtc-adapter");

var _canvas = require("canvas");

var _freeice = _interopRequireDefault(require("freeice"));

var _merge = _interopRequireDefault(require("merge"));

var _uuid = require("uuid");

var _wrtc = require("wrtc");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classPrivateMethodGet(receiver, privateSet, fn) { if (!privateSet.has(receiver)) { throw new TypeError("attempted to get private field on non-instance"); } return fn; }

function _classPrivateFieldGet(receiver, privateMap) { var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "get"); return _classApplyDescriptorGet(receiver, descriptor); }

function _classApplyDescriptorGet(receiver, descriptor) { if (descriptor.get) { return descriptor.get.call(receiver); } return descriptor.value; }

function _classPrivateFieldSet(receiver, privateMap, value) { var descriptor = _classExtractFieldDescriptor(receiver, privateMap, "set"); _classApplyDescriptorSet(receiver, descriptor, value); return value; }

function _classExtractFieldDescriptor(receiver, privateMap, action) { if (!privateMap.has(receiver)) { throw new TypeError("attempted to " + action + " private field on non-instance"); } return privateMap.get(receiver); }

function _classApplyDescriptorSet(receiver, descriptor, value) { if (descriptor.set) { descriptor.set.call(receiver, value); } else { if (!descriptor.writable) { throw new TypeError("attempted to set read only private field"); } descriptor.value = value; } }

const recursive = _merge.default.recursive.bind(undefined, true);

const MEDIA_CONSTRAINTS = {
  audio: true,
  video: {
    width: 640,
    framerate: 15
  }
};
/**
 * Apply an optional callback to a promise and return the new one
 *
 * @param Promise promise
 * @param {} [callback]
 */

function asCallback(promise, callback) {
  if (callback) return promise.then(callback.bind(null, null), callback);
  return promise;
}

function filterTracksType({
  track: {
    kind
  }
}) {
  return this.kind === kind;
}

function replaceTrack(sender) {
  const {
    track
  } = sender;
  return sender.replaceTrack(this).then(track === null || track === void 0 ? void 0 : track.stop.bind(track));
}

function replaceTracks(track) {
  return this.filter(filterTracksType, track).map(replaceTrack, track);
}
/* Simulcast utilities */


function removeFIDFromOffer(sdp) {
  const n = sdp.indexOf("a=ssrc-group:FID");
  if (n === -1) return sdp;
  return sdp.slice(0, n);
}

function getFirstVideoTrack(stream) {
  // Ensure all video tracks except first one and all audio tracks are stopped
  for (const track of stream.getAudioTracks()) track.stop();

  const [result, ...tracks] = stream.getVideoTracks();

  for (const track of tracks) track.stop();

  return result;
}

function getMediaEnabled(type) {
  // [ToDo] Should return undefined if not all tracks have the same value?
  const senders = this.peerConnection.getSenders();
  if (!senders.length) return;
  type = type.toLowerCase();

  for (const {
    track
  } of senders) if (track.kind === type && !track.enabled) return false;

  return true;
}

function setMediaEnabled(type, value) {
  type = type.toLowerCase();

  for (const {
    track
  } of this.peerConnection.getSenders()) if (track.kind === type) track.enabled = value;
}
/**
 * Wrapper object of an {RTCPeerConnection}. This object is aimed to simplify
 * the development of WebRTC-based applications.
 */


var _audioStream = /*#__PURE__*/new WeakMap();

var _candidategatheringdone = /*#__PURE__*/new WeakMap();

var _candidatesQueueOut = /*#__PURE__*/new WeakMap();

var _dataChannelConfig = /*#__PURE__*/new WeakMap();

var _dataChannels = /*#__PURE__*/new WeakMap();

var _id = /*#__PURE__*/new WeakMap();

var _interop = /*#__PURE__*/new WeakMap();

var _dataChannel = /*#__PURE__*/new WeakMap();

var _lastFrame = /*#__PURE__*/new WeakMap();

var _logger = /*#__PURE__*/new WeakMap();

var _mediaConstraints = /*#__PURE__*/new WeakMap();

var _mode = /*#__PURE__*/new WeakMap();

var _onnegotiationneeded = /*#__PURE__*/new WeakMap();

var _ontrack = /*#__PURE__*/new WeakMap();

var _peerConnection = /*#__PURE__*/new WeakMap();

var _peerconnectionConfiguration = /*#__PURE__*/new WeakMap();

var _ready = /*#__PURE__*/new WeakMap();

var _sendSource = /*#__PURE__*/new WeakMap();

var _simulcast = /*#__PURE__*/new WeakMap();

var _usePlanB = /*#__PURE__*/new WeakMap();

var _videoStream = /*#__PURE__*/new WeakMap();

var _videoSink = /*#__PURE__*/new WeakMap();

var _getMedia = /*#__PURE__*/new WeakSet();

var _getSimulcastInfo = /*#__PURE__*/new WeakSet();

var _initPeerConnection = /*#__PURE__*/new WeakSet();

var _onConnectionStateChange = /*#__PURE__*/new WeakMap();

var _onFrame = /*#__PURE__*/new WeakMap();

var _onIcecandidate = /*#__PURE__*/new WeakMap();

var _onNewListener = /*#__PURE__*/new WeakSet();

var _replaceTrack = /*#__PURE__*/new WeakMap();

var _setLocalDescription = /*#__PURE__*/new WeakMap();

var _setVideoStream = /*#__PURE__*/new WeakSet();

var _setRemoteVideo = /*#__PURE__*/new WeakMap();

var _start = /*#__PURE__*/new WeakMap();

class WebRtcPeerCore extends _events.default {
  /**
   * @constructor module:kurentoUtils.WebRtcPeer
   *
   * @param {String} mode Mode in which the PeerConnection will be configured.
   *  Valid values are: 'recv', 'send', and 'sendRecv'
   * @param {Object} [options]
   * @param {MediaStream} [options.audioStream] Stream to be used as second
   *  source (typically for audio) for localVideo and to be added as stream to
   *  the {RTCPeerConnection}
   * @param {MediaStream} [options.videoStream] Stream to be used as primary
   *  source (typically video and audio, or only video if combined with
   *  audioStream) for localVideo and to be added as stream to the
   *  {RTCPeerConnection}
   * @param {} [callback] Deprecated
   */
  constructor(mode, _options, callback) {
    if (_options instanceof Function) {
      callback = _options;
      _options = undefined;
    }

    const {
      audioStream,
      configuration,
      dataChannelConfig = {},
      dataChannels,
      freeice: freeiceOpts,
      id: _id2 = (0, _uuid.v4)(),
      logger = console,
      mediaConstraints = MEDIA_CONSTRAINTS,
      oncandidategatheringdone,
      onicecandidate,
      onnegotiationneeded,
      onstreamended,
      ontrack,
      peerConnection,
      sendSource = 'webcam',
      simulcast,
      usePlanB,
      videoStream: _videoStream2
    } = _options || {};
    super();

    _setVideoStream.add(this);

    _onNewListener.add(this);

    _initPeerConnection.add(this);

    _getSimulcastInfo.add(this);

    _getMedia.add(this);

    _audioStream.set(this, {
      writable: true,
      value: void 0
    });

    _candidategatheringdone.set(this, {
      writable: true,
      value: void 0
    });

    _candidatesQueueOut.set(this, {
      writable: true,
      value: []
    });

    _dataChannelConfig.set(this, {
      writable: true,
      value: void 0
    });

    _dataChannels.set(this, {
      writable: true,
      value: void 0
    });

    _id.set(this, {
      writable: true,
      value: void 0
    });

    _interop.set(this, {
      writable: true,
      value: void 0
    });

    _dataChannel.set(this, {
      writable: true,
      value: void 0
    });

    _lastFrame.set(this, {
      writable: true,
      value: void 0
    });

    _logger.set(this, {
      writable: true,
      value: void 0
    });

    _mediaConstraints.set(this, {
      writable: true,
      value: void 0
    });

    _mode.set(this, {
      writable: true,
      value: void 0
    });

    _onnegotiationneeded.set(this, {
      writable: true,
      value: void 0
    });

    _ontrack.set(this, {
      writable: true,
      value: void 0
    });

    _peerConnection.set(this, {
      writable: true,
      value: void 0
    });

    _peerconnectionConfiguration.set(this, {
      writable: true,
      value: void 0
    });

    _ready.set(this, {
      writable: true,
      value: void 0
    });

    _sendSource.set(this, {
      writable: true,
      value: void 0
    });

    _simulcast.set(this, {
      writable: true,
      value: void 0
    });

    _usePlanB.set(this, {
      writable: true,
      value: void 0
    });

    _videoStream.set(this, {
      writable: true,
      value: void 0
    });

    _videoSink.set(this, {
      writable: true,
      value: void 0
    });

    _onConnectionStateChange.set(this, {
      writable: true,
      value: () => {
        _classPrivateFieldGet(this, _logger).debug('onConnectionStateChange', _classPrivateFieldGet(this, _peerConnection).connectionState);

        switch (_classPrivateFieldGet(this, _peerConnection).connectionState) {
          case "connected": // The connection has become fully connected

          case "disconnected": // One or more transports has terminated

          case "failed":
            // unexpectedly or in an error
            break;

          case "closed":
            // The connection has been closed
            _classPrivateFieldSet(this, _peerConnection, new _wrtc.RTCPeerConnection(_classPrivateFieldGet(this, _peerconnectionConfiguration)));

            _classPrivateMethodGet(this, _initPeerConnection, _initPeerConnection2).call(this);

            break;
        }
      }
    });

    _onFrame.set(this, {
      writable: true,
      value: ({
        frame
      }) => _classPrivateFieldSet(this, _lastFrame, frame)
    });

    _onIcecandidate.set(this, {
      writable: true,
      value: ({
        candidate
      }) => {
        _classPrivateFieldGet(this, _logger).debug('onIcecandidate', candidate);

        if (this.listenerCount('icecandidate') || this.listenerCount('candidategatheringdone')) {
          if (candidate) {
            this.emit('icecandidate', candidate);

            _classPrivateFieldSet(this, _candidategatheringdone, false);
          } else if (!_classPrivateFieldGet(this, _candidategatheringdone)) {
            this.emit('candidategatheringdone');

            _classPrivateFieldSet(this, _candidategatheringdone, true);
          }
        } // Not listening to 'icecandidate' or 'candidategatheringdone' events,
        // queue the candidate until one of them is listened
        else if (!_classPrivateFieldGet(this, _candidategatheringdone)) {
            _classPrivateFieldGet(this, _candidatesQueueOut).push(candidate);

            if (!candidate) _classPrivateFieldSet(this, _candidategatheringdone, true);
          }
      }
    });

    _replaceTrack.set(this, {
      writable: true,
      value: (track = null) => {
        let senders = this.peerConnection.getSenders();
        if (track) senders = senders.filter(filterTracksType, track);
        return Promise.all(senders.map(replaceTrack, track));
      }
    });

    _setLocalDescription.set(this, {
      writable: true,
      value: localDescription => {
        if (_classPrivateFieldGet(this, _simulcast)) if (!_classPrivateFieldGet(this, _usePlanB)) _classPrivateFieldGet(this, _logger).warn('Simulcast is only available in Chrome browser.');else {
          _classPrivateFieldGet(this, _logger).debug('Adding multicast info');

          localDescription = new _wrtc.RTCSessionDescription({
            type: localDescription.type,
            sdp: removeFIDFromOffer(localDescription.sdp) + _classPrivateMethodGet(this, _getSimulcastInfo, _getSimulcastInfo2).call(this, _classPrivateFieldGet(this, _videoStream))
          });
        }
        return _classPrivateFieldGet(this, _peerConnection).setLocalDescription(localDescription);
      }
    });

    _setRemoteVideo.set(this, {
      writable: true,
      value: () => {
        if (_classPrivateFieldGet(this, _videoSink)) {
          _classPrivateFieldGet(this, _videoSink).stop();

          _classPrivateFieldGet(this, _videoSink).removeEventListener('frame', _classPrivateFieldGet(this, _onFrame));
        }

        this.emit('setRemoteVideo');
        if (!_wrtc.nonstandard) return;
        const [receiver] = this.peerConnection.getReceivers().filter(filterTracksType, {
          kind: 'video'
        });

        if (receiver) {
          _classPrivateFieldSet(this, _videoSink, new _wrtc.nonstandard.RTCVideoSink(receiver.track));

          _classPrivateFieldGet(this, _videoSink).addEventListener('frame', _classPrivateFieldGet(this, _onFrame));

          return;
        }

        _classPrivateFieldSet(this, _lastFrame, null);

        _classPrivateFieldSet(this, _videoSink, null);
      }
    });

    _start.set(this, {
      writable: true,
      value: () => {
        if (_classPrivateFieldGet(this, _peerConnection).connectionState === 'closed') throw new Error('The peer connection object is in `closed` state. This ' + 'is most likely due to an invocation of the `dispose` method before ' + 'accepting in the dialogue');
        const self = this; // TODO maybe this can be deleted?

        function streamEndedListener() {
          self.emit('streamended', this);
        }

        if (_classPrivateFieldGet(this, _videoStream)) {
          _classPrivateFieldGet(this, _videoStream).addEventListener('ended', streamEndedListener);

          for (const track of _classPrivateFieldGet(this, _videoStream).getTracks()) _classPrivateFieldGet(this, _peerConnection).addTrack(track);
        }

        if (_classPrivateFieldGet(this, _audioStream)) {
          _classPrivateFieldGet(this, _audioStream).addEventListener('ended', streamEndedListener);

          for (const track of _classPrivateFieldGet(this, _audioStream).getTracks()) _classPrivateFieldGet(this, _peerConnection).addTrack(track);
        }
      }
    });

    _classPrivateFieldSet(this, _audioStream, audioStream);

    _classPrivateFieldSet(this, _dataChannelConfig, dataChannelConfig);

    _classPrivateFieldSet(this, _dataChannels, dataChannels);

    _classPrivateFieldSet(this, _id, _id2);

    _classPrivateFieldSet(this, _logger, logger);

    _classPrivateFieldSet(this, _mediaConstraints, mediaConstraints);

    _classPrivateFieldSet(this, _mode, mode);

    _classPrivateFieldSet(this, _onnegotiationneeded, onnegotiationneeded);

    _classPrivateFieldSet(this, _ontrack, ontrack);

    _classPrivateFieldSet(this, _peerConnection, peerConnection);

    _classPrivateFieldSet(this, _sendSource, sendSource);

    _classPrivateFieldSet(this, _simulcast, simulcast);

    _classPrivateFieldSet(this, _usePlanB, usePlanB);

    _classPrivateFieldSet(this, _videoStream, _videoStream2);

    _classPrivateFieldSet(this, _peerconnectionConfiguration, recursive({
      iceServers: (0, _freeice.default)(freeiceOpts)
    }, configuration));

    if (usePlanB) _classPrivateFieldGet(this, _logger).debug(name + ": using SDP PlanB");
    this.on('newListener', _classPrivateMethodGet(this, _onNewListener, _onNewListener2));
    if (oncandidategatheringdone) this.on('candidategatheringdone', oncandidategatheringdone);
    if (onicecandidate) this.on('icecandidate', onicecandidate);
    if (onstreamended) this.on('streamended', onstreamended); // Init PeerConnection

    if (!_classPrivateFieldGet(this, _peerConnection)) {
      _classPrivateFieldSet(this, _peerConnection, new _wrtc.RTCPeerConnection(_classPrivateFieldGet(this, _peerconnectionConfiguration)));
    }

    _classPrivateMethodGet(this, _initPeerConnection, _initPeerConnection2).call(this);

    asCallback(this.ready, callback);
  } //
  // Public API
  //


  get audioEnabled() {
    return getMediaEnabled.call(this, 'Audio');
  }

  set audioEnabled(value) {
    setMediaEnabled.call(this, 'Audio', value);
  }
  /**
   * @member {(external:ImageData|undefined)} currentFrame
   */


  get currentFrame() {
    if (!_wrtc.nonstandard) throw new Error('Stream-based `currentFrame` only available in Node.js');

    const lastFrame = _classPrivateFieldGet(this, _lastFrame);

    if (!lastFrame) throw new Error('No remote video stream available');
    const {
      height,
      width
    } = lastFrame;
    const rgba = new Uint8ClampedArray(width * height * 4);
    const rgbaFrame = (0, _canvas.createImageData)(rgba, width, height);

    _wrtc.nonstandard.i420ToRgba(lastFrame, rgbaFrame);

    const canvas = (0, _canvas.createCanvas)(width, height);
    canvas.getContext('2d').putImageData(rgbaFrame, 0, 0);
    return canvas;
  }

  get dataChannel() {
    return _classPrivateFieldGet(this, _dataChannel);
  }

  get enabled() {
    return this.audioEnabled && this.videoEnabled;
  }

  set enabled(value) {
    this.audioEnabled = this.videoEnabled = value;
  }

  get id() {
    return _classPrivateFieldGet(this, _id);
  }

  get localSessionDescriptor() {
    return _classPrivateFieldGet(this, _peerConnection).localDescription;
  }

  get peerConnection() {
    return _classPrivateFieldGet(this, _peerConnection);
  }

  get ready() {
    return _classPrivateFieldGet(this, _ready);
  }

  get remoteSessionDescriptor() {
    return _classPrivateFieldGet(this, _peerConnection).remoteDescription;
  }

  get videoEnabled() {
    return getMediaEnabled.call(this, 'Video');
  }

  set videoEnabled(value) {
    setMediaEnabled.call(this, 'Video', value);
  }

  get videoStream() {
    return _classPrivateFieldGet(this, _videoStream);
  }
  /**
   * Callback function invoked when an ICE candidate is received. Developers are
   * expected to invoke this function in order to complete the SDP negotiation.
   *
   * @function module:kurentoUtils.WebRtcPeer.prototype.addIceCandidate
   *
   * @param candidate - Literal object with the ICE candidate description
   * @param [callback] - Deprecated: Called when the ICE candidate has been added
   *
   * @returns Promise
   */


  addIceCandidate(candidate, callback) {
    _classPrivateFieldGet(this, _logger).debug('Remote ICE candidate received', candidate);

    return asCallback(_classPrivateFieldGet(this, _peerConnection).addIceCandidate(candidate), callback);
  }
  /**
   * @description This method frees the resources used by WebRtcPeer.
   *
   * @function module:kurentoUtils.WebRtcPeer.prototype.dispose
   */


  dispose() {
    _classPrivateFieldGet(this, _logger).debug('Disposing WebRtcPeerCore');

    const pc = _classPrivateFieldGet(this, _peerConnection);

    if (pc.connectionState === 'closed') return;

    try {
      pc.close();
    } catch (err) {
      _classPrivateFieldGet(this, _logger).warn('Exception disposing webrtc peer:', err);
    }

    if (_classPrivateFieldGet(this, _audioStream)) for (const track of _classPrivateFieldGet(this, _audioStream).getTracks()) track.stop();
    if (_classPrivateFieldGet(this, _videoStream)) for (const track of _classPrivateFieldGet(this, _videoStream).getTracks()) track.stop();
    this.removeAllListeners();
  }
  /**
   * @param [callback] - Deprecated
   *
   * @returns {Promise}
   */


  generateOffer(callback) {
    switch (_classPrivateFieldGet(this, _mode)) {
      case 'recvonly':
        {
          /* Add reception tracks on the RTCPeerConnection. Send tracks are
           * unconditionally added to "sendonly" and "sendrecv" modes, in the
           * constructor's "start()" method, but nothing is done for "recvonly".
           *
           * Here, we add new transceivers to receive audio and/or video, so the
           * SDP Offer that will be generated by the PC includes these medias
           * with the "a=recvonly" attribute.
           */
          const useAudio = typeof _classPrivateFieldGet(this, _mediaConstraints).audio === 'boolean' ? _classPrivateFieldGet(this, _mediaConstraints).audio : true;
          const useVideo = typeof _classPrivateFieldGet(this, _mediaConstraints).video === 'boolean' ? _classPrivateFieldGet(this, _mediaConstraints).video : true;

          if (useAudio) {
            _classPrivateFieldGet(this, _peerConnection).addTransceiver('audio', {
              direction: 'recvonly'
            });
          }

          if (useVideo) {
            _classPrivateFieldGet(this, _peerConnection).addTransceiver('video', {
              direction: 'recvonly'
            });
          }
        }
        break;

      case 'sendonly':
        /* The constructor's "start()" method already added any available track,
         * which by default creates Transceiver with "sendrecv" direction.
         *
         * Here, we set all transceivers to only send audio and/or video, so
         * the SDP Offer that will be generated by the PC includes these medias
         * with the "a=sendonly" attribute.
         */
        for (const transceiver of _classPrivateFieldGet(this, _peerConnection).getTransceivers()) transceiver.direction = "sendonly";

    }

    const promise = _classPrivateFieldGet(this, _peerConnection).createOffer().then(_classPrivateFieldGet(this, _setLocalDescription)).then(() => {
      const {
        localDescription
      } = _classPrivateFieldGet(this, _peerConnection);

      _classPrivateFieldGet(this, _logger).debug('Local description set\n', localDescription.sdp);

      return localDescription.sdp;
    });

    return asCallback(promise, callback);
  }
  /**
   *
   * @param {Integer} [index]
   */


  getReceiver(index = 0) {
    return _classPrivateFieldGet(this, _peerConnection).getReceivers()[index];
  }
  /**
   *
   * @param {Integer} [index]
   */


  getSender(index = 0) {
    return _classPrivateFieldGet(this, _peerConnection).getSenders()[index];
  }
  /**
   * Callback function invoked when a SDP answer is received
   *
   * Developers are expected to invoke this function in order to complete the
   * SDP negotiation.
   *
   * @function module:kurentoUtils.WebRtcPeer.prototype.processAnswer
   *
   * @param sdp - Description of sdpAnswer
   * @param {} [callback] - Deprecated: Invoked after the SDP answer is
   *  processed, or there is an error
   *
   * @returns {Promise}
   */


  processAnswer(sdp, callback) {
    if (_classPrivateFieldGet(this, _peerConnection).connectionState === 'closed') return Promise.reject(new Error('PeerConnection is closed'));
    const answer = new _wrtc.RTCSessionDescription({
      type: 'answer',
      sdp
    });

    const promise = _classPrivateFieldGet(this, _peerConnection).setRemoteDescription(answer).then(_classPrivateFieldGet(this, _setRemoteVideo));

    return asCallback(promise, callback);
  }
  /**
   * Callback function invoked when a SDP offer is received
   *
   * Developers are expected to invoke this function in order to complete the
   * SDP negotiation.
   *
   * @function module:kurentoUtils.WebRtcPeer.prototype.processOffer
   *
   * @param sdp - Description of sdpOffer
   * @param {} [callback] - Deprecated: Called when the remote description has
   *  been set successfully
   *
   * @returns {Promise}
   */


  processOffer(sdp, callback) {
    if (_classPrivateFieldGet(this, _peerConnection).connectionState === 'closed') return Promise.reject(new Error('PeerConnection is closed'));
    const offer = new _wrtc.RTCSessionDescription({
      type: 'offer',
      sdp
    });

    const promise = _classPrivateFieldGet(this, _peerConnection).setRemoteDescription(offer).then(_classPrivateFieldGet(this, _setRemoteVideo)).then(_classPrivateFieldGet(this, _peerConnection).createAnswer.bind(_classPrivateFieldGet(this, _peerConnection))).then(_classPrivateFieldGet(this, _setLocalDescription)).then(() => _classPrivateFieldGet(this, _peerConnection).localDescription.sdp);

    return asCallback(promise, callback);
  }
  /**
   * Fully replace the sending stream without re-negotiation
   *
   * @param {MediaStream} stream
   *
   * @returns {Promise}
   */


  replaceStream(stream) {
    // Replace local video
    if (_classPrivateFieldGet(this, _videoStream)) for (const track of _classPrivateFieldGet(this, _videoStream).getTracks()) track.stop();

    _classPrivateMethodGet(this, _setVideoStream, _setVideoStream2).call(this, stream); // Replace senders


    const senders = this.peerConnection.getSenders();
    return Promise.all(stream.getTracks().flatMap(replaceTracks, senders));
  }
  /**
   * Replace the video track in the sending stream without re-negotiation
   *
   * @param {MediaTrack|String} track
   *
   * @returns {Promise}
   */


  replaceTrack(track) {
    const promise = typeof track === 'string' ? _classPrivateMethodGet(this, _getMedia, _getMedia2).call(this, track).then(getFirstVideoTrack) : Promise.resolve(track);
    return promise.then(_classPrivateFieldGet(this, _replaceTrack));
  }
  /**
   * Send a message using the DataChannel instance
   *
   * @param {*} data
   */


  send(data) {
    var _classPrivateFieldGet2;

    if (((_classPrivateFieldGet2 = _classPrivateFieldGet(this, _dataChannel)) === null || _classPrivateFieldGet2 === void 0 ? void 0 : _classPrivateFieldGet2.readyState) === 'open') return _classPrivateFieldGet(this, _dataChannel).send(data);

    _classPrivateFieldGet(this, _logger).warn('Trying to send data over a non-existing or closed data channel');
  } //
  // Private API
  //


} // https://github.com/Automattic/node-canvas/issues/487


exports.WebRtcPeerCore = exports.default = WebRtcPeerCore;

function _getMedia2(track) {
  let method = 'getUserMedia';

  let constraints = _classPrivateFieldGet(this, _mediaConstraints);

  if (track === 'screen') {
    method = 'getDisplayMedia';
    constraints = {
      audio: false,
      video: true
    };
  }

  return _wrtc.mediaDevices[method](constraints).then(stream => {
    if (_classPrivateFieldGet(this, _videoStream)) {
      for (const track of _classPrivateFieldGet(this, _videoStream).getVideoTracks()) track.stop();

      if (track === 'screen') for (const track of _classPrivateFieldGet(this, _videoStream).getAudioTracks()) stream.addTrack(track);
    }

    _classPrivateMethodGet(this, _setVideoStream, _setVideoStream2).call(this, stream);

    return stream;
  });
}

function _getSimulcastInfo2(videoStream) {
  const videoTracks = videoStream.getVideoTracks();

  if (!videoTracks.length) {
    _classPrivateFieldGet(this, _logger).warn('No video tracks available in the video stream');

    return '';
  }

  const [{
    id
  }] = videoTracks;
  return ['a=x-google-flag:conference', 'a=ssrc-group:SIM 1 2 3', 'a=ssrc:1 cname:localVideo', 'a=ssrc:1 msid:' + videoStream.id + ' ' + id, 'a=ssrc:1 mslabel:' + videoStream.id, 'a=ssrc:1 label:' + id, 'a=ssrc:2 cname:localVideo', 'a=ssrc:2 msid:' + videoStream.id + ' ' + id, 'a=ssrc:2 mslabel:' + videoStream.id, 'a=ssrc:2 label:' + id, 'a=ssrc:3 cname:localVideo', 'a=ssrc:3 msid:' + videoStream.id + ' ' + id, 'a=ssrc:3 mslabel:' + videoStream.id, 'a=ssrc:3 label:' + id, ''].join('\n');
}

function _initPeerConnection2() {
  if (_classPrivateFieldGet(this, _dataChannels)) {
    const {
      id = `WebRtcPeer-${_classPrivateFieldGet(this, _id)}`,
      onbufferedamountlow,
      onclose,
      onerror = _classPrivateFieldGet(this, _logger).error,
      onmessage,
      onopen,
      options
    } = _classPrivateFieldGet(this, _dataChannelConfig);

    _classPrivateFieldSet(this, _dataChannel, _classPrivateFieldGet(this, _peerConnection).createDataChannel(id, options));

    _classPrivateFieldGet(this, _dataChannel).addEventListener('open', onopen);

    _classPrivateFieldGet(this, _dataChannel).addEventListener('close', onclose);

    _classPrivateFieldGet(this, _dataChannel).addEventListener('message', onmessage);

    _classPrivateFieldGet(this, _dataChannel).addEventListener('bufferedamountlow', onbufferedamountlow);

    _classPrivateFieldGet(this, _dataChannel).addEventListener('error', onerror);
  }

  _classPrivateFieldGet(this, _peerConnection).addEventListener('connectionstatechange', _classPrivateFieldGet(this, _onConnectionStateChange));

  _classPrivateFieldGet(this, _peerConnection).addEventListener('icecandidate', _classPrivateFieldGet(this, _onIcecandidate));

  if (_classPrivateFieldGet(this, _onnegotiationneeded)) _classPrivateFieldGet(this, _peerConnection).addEventListener('negotiationneeded', _classPrivateFieldGet(this, _onnegotiationneeded));
  if (_classPrivateFieldGet(this, _ontrack)) _classPrivateFieldGet(this, _peerConnection).addEventListener('track', _classPrivateFieldGet(this, _ontrack));
  let promise;

  if (_classPrivateFieldGet(this, _mode) === 'recvonly' || _classPrivateFieldGet(this, _videoStream) || _classPrivateFieldGet(this, _audioStream)) {
    this.emit('setLocalVideo');
    promise = Promise.resolve();
  } else promise = _classPrivateMethodGet(this, _getMedia, _getMedia2).call(this, _classPrivateFieldGet(this, _sendSource));

  _classPrivateFieldSet(this, _ready, promise.then(_classPrivateFieldGet(this, _start)));
}

function _onNewListener2(event, listener) {
  const iscandidategatheringdone = event === 'candidategatheringdone';
  if (iscandidategatheringdone && event !== 'icecandidate') return;
  let candidate;

  while (candidate = _classPrivateFieldGet(this, _candidatesQueueOut).shift()) if (!candidate === iscandidategatheringdone) listener(candidate);
}

function _setVideoStream2(stream) {
  _classPrivateFieldSet(this, _videoStream, stream);

  this.emit('setLocalVideo');
}